% This file explains how use the provided data and code to estimate a
% return predictability regression using WLS-EV
%
% It should be relatively easy to apply using your own x variable, and/or a
% different time-series of returns
%

%% -----------------------------------------------------------------------
% EXAMPLE 1: Using WLS-EV for a new x variable without re-estimating the
% first stage volatility forecast, no overlapping lhs or bootstrapping
% 
% Requirements: 1) wlsev.m and olsgmm.m must be in Matlab's path
%               2) you must have your own monthly returns and x variable
%               values from elsewhere
cd('C:\Dropbox\Archives\GLS\For website\Code\To use WLS-EV\'); 
clear variables; close all; clc; 

% load provided .csv data 
% monthly version
load 'Johnson2018_monthly_1927_2015.mat';
sigsq_data = Johnson2018_monthly_1927_2015; clear Johnson2018_monthly_1927_2015; 

% daily version -- you can figure the rest out on your own
% load 'Johnson2018_monthly_1927_2015.mat';
% sigsq_data = Johnson2018_monthly_1927_2015; clear Johnson2018_monthly_1927_2015; 

% load return and x data
load 'example_rx.mat'; 

% compute lhs and rhs
% use next-month returns for lhs
lhs = example_rx.mktrf(2:end); 
T = length(lhs); 

% constant and x for rhs
rhs = [ ones(T,1) example_rx.mktrf(1:end-1) ]; 

[coeff_1, SE_1, rsq_1, rsq_adj_1, v_1] = wlsev(lhs, ... % first input is lhs
    rhs, ... % second input rhs
    sigsq_data.sigsq_rv(1:end-1), ... % third input is predicted next-period variance (sigsq_rv from paper)
    3, ... % fourth input is number of lags to use for standard errors
    1); % fifth input is weight to use for standard errors (1=newey-west, 0=hansen hodrick)

%% -----------------------------------------------------------------------
% EXAMPLE 2: Using WLS-EV for a new x variable and re-estimating the
% first-stage regression
% 
% Requirements: 1) wlsev.m and olsgmm.m must be in Matlab's path
%               2) you must have your own monthly returns, x variable, and
%               variance predictors from elsewhere
cd('C:\Dropbox\Archives\GLS\For website\Code\To use WLS-EV\'); 
clear variables; close all; clc; 

% load dataset containing realized variance estimates and variance
% forecasters
load 'example_sigmas.mat'; 

% load return and x data
load 'example_rx.mat'; 

% compute lhs and rhs
% use next-month returns for lhs
lhs = example_rx.mktrf(2:end); 
T = length(lhs); 

% constant and x for rhs
rhs = [ ones(T,1) example_rx.mktrf(1:end-1) ]; 

% sigsq are all the ex-ante variables you want to use to predict realized
% variance
sigsq = [ example_sigmas.rv_0(1:end-1)  example_sigmas.rv_m11_0(1:end-1) ]; % these are past-month and past-year variances

% realvar is realized within-period variance
realvar = example_sigmas.rv_0(2:end); 

[coeff_2, SE_2, rsq_2, rsq_adj_2, v_2] = wlsev(lhs, ... % first input is lhs
    rhs, ... % second input rhs
    sigsq, ... % third input is predicted next-period variance (sigsq_rv from paper)
    3, ... % fourth input is number of lags to use for standard errors
    1, ... % fifth input is weight to use for standard errors (1=newey-west, 0=hansen hodrick)
    realvar ); % sixth input is realized within-period variance.

% This should provide same answer as the first example since these are the
% variables I use to compute sigsq_rv in the paper. However, if I change
% sigsq to just use past-year variance:
sigsq = [ example_sigmas.rv_m11_0(1:end-1) ]; % these are past-month and past-year variances

[coeff_3, SE_3, rsq_3, rsq_adj_3, v_3] = wlsev(lhs, ... % first input is lhs
    rhs, ... % second input rhs
    sigsq, ... % third input is predicted next-period variance (sigsq_rv from paper)
    3, ... % fourth input is number of lags to use for standard errors
    1, ... % fifth input is weight to use for standard errors (1=newey-west, 0=hansen hodrick)
    realvar ); % sixth input is realized within-period variance.

% now the answer is slightly different


%% -----------------------------------------------------------------------
% EXAMPLE 3: Using WLS-EV for a new x variable without re-estimating the
% first stage volatility forecast, bootstrap standard errors with
% overlapping and stambaugh correction
% 
% Requirements: 1) wlsev.m, olsgmm.m, sim_returns.m, and olswlsbootstrap_keepxsig.m must be in matlab's path
%               2) you must have your own monthly returns and x variable
%               values from elsewhere to input
cd('C:\Dropbox\Archives\GLS\For website\Code\To use WLS-EV\'); 
clear variables; close all; clc; 

% load provided .csv data 
% monthly version
load 'Johnson2018_monthly_1927_2015.mat';
sigsq_data = Johnson2018_monthly_1927_2015; clear Johnson2018_monthly_1927_2015; 

% daily version -- you can figure the rest out on your own
% load 'Johnson2018_monthly_1927_2015.mat';
% sigsq_data = Johnson2018_monthly_1927_2015; clear Johnson2018_monthly_1927_2015; 

% load return and x data
load 'example_rx.mat'; 

% 4 data inputs required
in.r = example_rx.mktrf;            % one-period market returns
in.x = example_rx.x;                % contemporaneous x values
in.realvar = sigsq_data.rv;         % contemporaneous within-period realized returns
in.predvar = sigsq_data.sigsq_rv;   % this period's prediction of next-period variance

% parameters for the simulation
params.nsim = 100;              % keeping it to 100 so this runs quicklky
params.nfutperiods = 12;        % predict next-12 period returns
params.adjuststambaugh = true;  % yes to the stambaugh (1999) correction
params.lhsmultiplier = 100;     % scale lhs up by 100 to make answer more readable
params.nlags = 6;               % number of lags for newey-west

[ ols_stats, wls_stats, ols_simcoeff, wls_simcoeff ] = olswlsbootstrap_keepxsig(in,params); 
    
% have a look at the outputs you get on your own, they are fairly
% self-explanitory






