function [coeff, SE, rsq, rsq_adj, v, firststage] = wlsev(lhs,rhs,sigsq,lags,weight,realvar)
% [coeff, SE, rsq, rsq_adj, v, firststage] = wlsev(lhs,rhs,sigsq,lags,weight,realvar)
% Requires lhs be T x 1, RHS be T x K
%
% INPUTS:
% lhs = a T x N vector of returns to predict. If N>1, predict each lhs col in
%       separate regressions with same rhs matrix
% rhs = a T x K matrix of predictors, including a constant if you want one
% sigsq = either a T x 1 vector of ex-ante variances to use as weights
% (computed using a first-stage regression elsewhere)
%       or a T x M matrix of variance instruments to use in a first-stage
%       regression predicting realvar using sigsq  and a constant
% lags = number of lags to use for SEs
%  weight: 1 for newey-west weighting 
%          0 for even weighting
%         -1 skips standard error computations. This speeds things up a lot
%
% (optional) realvar = a T x 1 vector of within-period realized variances
% we predict in the first-stage regression.
%
% Timing convention: rhs and sigsq observed at t, lhs and realvar observed
% in future, so we regress lhs as given on rhs (and realvar on sigsq)
% without any timing adjustments
%
% OUTPUTS:
% coeff = wls-ev point estimates
% SE = wls-ev asymptotic standard errors
% rsq = R squared
% rsq_adj = Adjusted R squared
% v = covariance matrix of the point estimates
% firststage = struct containing fields for the first-stage regression
% coefficients (.predvar_coeff) and the time-series of predicted variances
% (.predvar). 

T = size(lhs,1); 
N = size(lhs,2); 
K = size(rhs,2); 

% if we receive realvar, do first-stage regression
if( nargin == 6)

    % % First-stage regression: realvar = a + b sigsq + delta, constrained so
    % sigsqhat > 0

    % First stage regression
    nsigsq = size(sigsq,2); 
    options = optimset('algorithm','interior-point','display','none'); coeff_0 = [mean(realvar); zeros(nsigsq,1)]; 
    predvar_coeff = lsqlin([ones(T,1) sigsq ],realvar,[],[],[],[],zeros(nsigsq+1,1),[],coeff_0,options); 
    predvar = [ones(T,1) sigsq ]*predvar_coeff; 

    firststage.predvar_coeff = predvar_coeff; 
    firststage.predvar = predvar; 

    % Second-stage regression: lhs = alpha + beta x + epsilon, estimated to
    % minimize sum (epsilon^2)/predvar^2
    [coeff, SE, ~, ~, v, ~] = olsgmm(lhs./sqrt(predvar),rhs ./ (sqrt(predvar)*ones(1,K)),lags,weight); 

% we did not receive realvar, use sigsq direclty as weights
else
    firststage = 'not used'; 

    [coeff, SE, ~, ~, v, ~] = olsgmm(lhs./(sqrt(sigsq)*ones(1,N)),rhs ./ (sqrt(sigsq)*ones(1,K)),lags,weight); 
end

% Now need to compute R^2. Can't use above since it modifies LHS and RHS
epsilon = lhs-rhs*coeff;
meanepssq = mean(epsilon.^2);
vary = var(lhs,1);

rsq = (1-meanepssq/vary)';
rsq_adj = (1 - (meanepssq./vary)*(T-1)/(T-K));